@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2009 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -54,11 +54,10 @@ int num_processes=0;
 static int *ipc_socket, *busy;
 #endif /* USE_PTHREAD */
 
-#define HAVE_MSGHDR_MSG_CONTROL 1
 
 void libwrap_init(int num) {
 #ifdef USE_PTHREAD
-    int i, rfd, result;
+    int i, j, rfd, result;
     char servname[STRLEN];
 
     num_processes=num;
@@ -80,9 +79,17 @@ void libwrap_init(int num) {
             ioerror("fork");
             die(1);
         case  0:    /* child */
-            close(ipc_socket[2*i]); /* server side */
+            drop_privileges(); /* libwrap processes are not chrooted */
+            close(0); /* stdin */
+            close(1); /* stdout */
+            if(!options.option.foreground) /* for logging in read_fd */
+                close(2); /* stderr */
+            close(ipc_socket[2*i]); /* close server-side socket */
+            for(j=0; j<i; ++j) /* previously created client-side sockets */
+                close(ipc_socket[2*j+1]);
             while(1) { /* main libwrap client loop */
-                read_fd(ipc_socket[2*i+1], servname, STRLEN, &rfd);
+                if(read_fd(ipc_socket[2*i+1], servname, STRLEN, &rfd)<=0)
+                    _exit(0);
                 result=check_libwrap(servname, rfd);
                 write(ipc_socket[2*i+1], (u8 *)&result, sizeof(result));
                 if(rfd>=0)
@@ -90,9 +97,9 @@ void libwrap_init(int num) {
             }
         default:    /* parent */
 #ifdef FD_CLOEXEC
-            fcntl(ipc_socket[2*i], F_SETFD, FD_CLOEXEC); /* server side */
+            fcntl(ipc_socket[2*i], F_SETFD, FD_CLOEXEC); /* server-side socket */
 #endif
-            close(ipc_socket[2*i+1]); /* client side */
+            close(ipc_socket[2*i+1]); /* client-side socket */
         }
     }
 #endif /* USE_PTHREAD */
@@ -106,7 +113,6 @@ void auth_libwrap(CLI *c) {
     static pthread_mutex_t mutex=PTHREAD_MUTEX_INITIALIZER;
     static pthread_cond_t cond=PTHREAD_COND_INITIALIZER;
 
-
     if(num_processes) {
         s_log(LOG_DEBUG, "Waiting for a libwrap process");
 
