@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2009 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -41,6 +48,9 @@
 #ifndef EAI_NONAME
 #define EAI_NONAME 2
 #endif
+#ifndef EAI_SERVICE
+#define EAI_SERVICE 8
+#endif
 
 #if defined(USE_WIN32) && defined(USE_IPv6)
 /* rename some locally shadowed declarations */
@@ -72,10 +82,10 @@ static const char *s_gai_strerror(int);
 
 #ifndef HAVE_GETNAMEINFO
 #ifndef NI_NUMERICHOST
-#define NI_NUMERICHOST	2
+#define NI_NUMERICHOST  2
 #endif
 #ifndef NI_NUMERICSERV
-#define NI_NUMERICSERV	8
+#define NI_NUMERICSERV  8
 #endif
 static int getnameinfo(const struct sockaddr *, int,
     char *, int , char *, int , int );
@@ -117,8 +127,11 @@ int hostport2addrlist(SOCKADDR_LIST *addr_list,
 
     err=getaddrinfo(hostname, portname, &hints, &res);
     if(err) {
-        s_log(LOG_ERR, "Error resolving '%s': %s",
-            hostname, s_gai_strerror(err));
+        if(err==EAI_SERVICE)
+            s_log(LOG_ERR, "Unknown TCP service '%s'", portname);
+        else
+            s_log(LOG_ERR, "Error resolving '%s': %s",
+                hostname, s_gai_strerror(err));
         if(res)
             freeaddrinfo(res);
         return 0; /* Error */
@@ -306,7 +319,7 @@ static void freeaddrinfo(struct addrinfo *current) {
 #if defined(USE_WIN32) && !defined(_WIN32_WCE)
     if(s_freeaddrinfo) {
         s_freeaddrinfo(current);
-	return;
+    return;
     }
 #endif
     while(current) {
